@@ -3,11 +3,13 @@ import logging
 from django.db.models import Model
 from transliterate import translit
 
-from isc_common import setAttr
+from isc_common import setAttr, delAttr
+from isc_common.bit import TurnBitOn, TurnBitOff
 from isc_common.fields.code_field import CodeField
 from isc_common.fields.description_field import DescriptionField
 from isc_common.fields.name_field import NameField
 from isc_common.fields.related import ForeignKeyProtect
+from isc_common.http.DSRequest import DSRequest
 from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
 
 logger = logging.getLogger(__name__)
@@ -15,6 +17,17 @@ logger = logging.getLogger(__name__)
 
 class BaseRefQuerySet(AuditQuerySet):
 
+    def filter(self, *args, **kwargs):
+        key = [key for key in kwargs.keys() if key == 'full_name']
+        if len(key) == 0:
+            return super().filter(*args, **kwargs)
+        else:
+            full_name = kwargs.get('full_name')
+            res = self.get_full_name_id(None, full_name[1:])
+            delAttr(kwargs, 'full_name')
+            kwargs.setdefault('pk', res.pk)
+            return super().filter(*args, **kwargs)
+
     def get_full_name_id(self, parent_id, name):
         end = False
         if name.find('/') != -1:
@@ -40,10 +53,9 @@ class BaseRefQuerySet(AuditQuerySet):
         else:
             full_name = kwargs.get('full_name')
             if full_name == None:
-               full_name = ''
+                full_name = ''
             res = self.get_full_name_id(None, full_name[1:])
 
-
         return res
 
     def update(self, **kwargs):
@@ -62,8 +74,43 @@ class BaseRefQuerySet(AuditQuerySet):
 class BaseRefManager(AuditManager):
     def get_queryset(self):
         if self.alive_only:
-            return BaseRefQuerySet(model=self.model, alive_only=self.alive_only, enabledAll=self.enabledAll).filter(deleted_at=None)
-        return BaseRefQuerySet(model=self.model, alive_only=self.alive_only, enabledAll=self.enabledAll)
+            return BaseRefQuerySet(model=self.model, alive_only=self.alive_only).filter(deleted_at=None)
+        return BaseRefQuerySet(model=self.model, alive_only=False)
+
+
+class StatusBaseRefManager(BaseRefManager):
+    def createFromRequest(self, request):
+        request = DSRequest(request=request)
+        data = request.get_data()
+        _data = data.copy()
+
+        disabled = _data.get('disabled')
+        props = _data.get('props', 0)
+
+        delAttr(_data, 'disabled')
+        props = TurnBitOn(props, 0) if disabled else TurnBitOff(props, 0)
+        setAttr(_data, 'props', props)
+
+        super().create(**_data)
+        setAttr(data, 'props', props)
+        return data
+
+    def updateFromRequest(self, request, removed=None, function=None):
+        request = DSRequest(request=request)
+        data = request.get_data()
+        _data = data.copy()
+
+        disabled = _data.get('disabled')
+        props = _data.get('props', 0)
+
+        delAttr(_data, 'disabled')
+        delAttr(_data, 'id')
+        props = TurnBitOn(props, 0) if disabled else TurnBitOff(props, 0)
+        setAttr(_data, 'props', props)
+        setAttr(data, 'props', props)
+
+        super().filter(id=data.get('id')).update(**_data)
+        return data
 
 
 class BaseRef(AuditModel):
@@ -99,25 +146,37 @@ class BaseHierarcy(Model):
             return self.get_parent_name(item.parent, name)
         return name
 
+    def get_root(self, parent):
+
+        if not parent:
+            return self
+
+        if parent.parent:
+            return self.get_parent_name(parent.parent)
+        return parent
+
     @property
     def full_name(self):
         if not self.name:
             if self.code:
-               self.name = self.code
+                self.name = self.code
             else:
                 raise Exception('Name and Code not Exists')
 
-
         name = f'/{self.name}'
         return self.get_parent_name(self.parent, name)
 
     @property
     def full_name_id(self):
-        if not self.name:
+        if not self.code:
             return None
 
         return self.objects.get(full_name=self.full_name)
 
+    @property
+    def root(self):
+        return self.get_root(self.parent)
+
     objects = BaseRefManager()
     all_objects = BaseRefManager(alive_only=False)
 
